@@ -27,6 +27,7 @@
 #include "string.h"
 /*#include <stdlib.h>*/
 #include <linux/mman.h>
+#include <linux/stat.h>
 #include "hash.h"
 #include "linuxelf.h"
 #include "sysdep.h"
@@ -134,8 +135,8 @@ int _dl_unmap_cache(void)
 unsigned int _dl_error_number;
 unsigned int _dl_internal_error_number;
 
-struct elf_resolve * _dl_load_shared_library(struct elf_resolve * tpnt,
-	char * full_libname){
+struct elf_resolve * _dl_load_shared_library(int secure, 
+	struct elf_resolve * tpnt, char * full_libname) {
   char * pnt, *pnt1, *pnt2;
   struct elf_resolve *tpnt1 = NULL;
   char mylibname[2050];
@@ -161,7 +162,7 @@ struct elf_resolve * _dl_load_shared_library(struct elf_resolve * tpnt,
   if (libname != full_libname) {
 #ifdef IBCS_COMPATIBLE
     if (_dl_strncmp(full_libname, "/usr/lib/", 9) != 0) {
-      tpnt1 = _dl_load_elf_shared_library(full_libname, 0);
+      tpnt1 = _dl_load_elf_shared_library(secure, full_libname, 0);
     } else {
       pnt1 = "/usr/i486-sysv4/lib/";
       pnt = mylibname;
@@ -169,10 +170,10 @@ struct elf_resolve * _dl_load_shared_library(struct elf_resolve * tpnt,
       pnt1 = full_libname + 9;
       while(*pnt1) *pnt++ = *pnt1++;
       *pnt++ = 0;
-      tpnt1 = _dl_load_elf_shared_library(mylibname, 0);
+      tpnt1 = _dl_load_elf_shared_library(secure, mylibname, 0);
     }
 #else
-    tpnt1 = _dl_load_elf_shared_library(full_libname, 0);
+    tpnt1 = _dl_load_elf_shared_library(secure, full_libname, 0);
 #endif
     if (tpnt1)
       return tpnt1;
@@ -203,7 +204,7 @@ struct elf_resolve * _dl_load_shared_library(struct elf_resolve * tpnt,
 	  pnt = libname;
 	  while(*pnt) *pnt2++  = *pnt++;
 	  *pnt2++ = 0;
-	  tpnt1 = _dl_load_elf_shared_library(mylibname, 0);
+	  tpnt1 = _dl_load_elf_shared_library(secure, mylibname, 0);
 	  if(tpnt1) return tpnt1;
 	  if(*pnt1 == ':') pnt1++;
 	}
@@ -229,7 +230,7 @@ struct elf_resolve * _dl_load_shared_library(struct elf_resolve * tpnt,
       pnt = libname;
       while(*pnt) *pnt2++  = *pnt++;
       *pnt2++ = 0;
-      tpnt1 = _dl_load_elf_shared_library(mylibname, 0);
+      tpnt1 = _dl_load_elf_shared_library(secure, mylibname, 0);
       if(tpnt1) return tpnt1;
       if(*pnt1 == ':' || *pnt1 == ';') pnt1++;
     }
@@ -254,7 +255,7 @@ struct elf_resolve * _dl_load_shared_library(struct elf_resolve * tpnt,
       if ((libent[i].flags == LIB_ELF || 
 	   libent[i].flags == LIB_ELF_LIBC5) &&
 	  _dl_strcmp(libname, strs+libent[i].sooffset) == 0 &&
-	  (tpnt1 = _dl_load_elf_shared_library(strs+libent[i].liboffset, 0)))
+	  (tpnt1 = _dl_load_elf_shared_library(secure, strs+libent[i].liboffset, 0)))
 	return tpnt1;
     }
   }
@@ -271,7 +272,7 @@ struct elf_resolve * _dl_load_shared_library(struct elf_resolve * tpnt,
   pnt1 = libname;
   while(*pnt1) *pnt++ = *pnt1++;
   *pnt++ = 0;
-  tpnt1 = _dl_load_elf_shared_library(mylibname, 0);
+  tpnt1 = _dl_load_elf_shared_library(secure, mylibname, 0);
   if (tpnt1) return tpnt1;
   
 #ifndef IBCS_COMPATIBLE
@@ -283,7 +284,7 @@ struct elf_resolve * _dl_load_shared_library(struct elf_resolve * tpnt,
   pnt1 = libname;
   while(*pnt1) *pnt++ = *pnt1++;
   *pnt++ = 0;
-  tpnt1 = _dl_load_elf_shared_library(mylibname, 0);
+  tpnt1 = _dl_load_elf_shared_library(secure, mylibname, 0);
   if (tpnt1) return tpnt1;
 #endif
 
@@ -302,7 +303,8 @@ goof:
 
 extern _elf_rtbndr(void);
 
-struct elf_resolve * _dl_load_elf_shared_library(char * libname, int flag){
+struct elf_resolve * _dl_load_elf_shared_library(int secure, 
+        char * libname, int flag) {
   struct elfhdr * epnt;
   unsigned int dynamic_addr = 0;
   unsigned int dynamic_size = 0;
@@ -325,6 +327,15 @@ struct elf_resolve * _dl_load_elf_shared_library(char * libname, int flag){
   tpnt = _dl_check_hashed_files(libname);
   if(tpnt) return tpnt;
 
+  /* If we are in secure mode (i.e. a setu/gid binary using LD_PRELOAD),
+     we don't load the library if it isn't setuid. */
+
+  if (secure) {
+    struct stat st;
+    if (_dl_stat(libname, &st) || !(st.st_mode & S_ISUID))
+      return NULL;
+  }
+
   libaddr = 0;
   infile = _dl_open(libname, O_RDONLY);
   if(infile < 0)
