@@ -9,16 +9,16 @@
 #include <stdarg.h>
 #include <unistd.h>
 
-extern int vsprintf(char * buf, const char * fmt, va_list args);
+extern int vsnprintf(char * buf, int size, const char * fmt, va_list args);
 
 int fdprintf(int fd, const char *fmt, ...)
 {
 	va_list args;
 	int i;
-	char buf[1024];
+	char buf[2048];
 
 	va_start(args, fmt);
-	i=vsprintf(buf,fmt,args);
+	i=vsnprintf(buf,sizeof buf,fmt,args);
 	va_end(args);
 	write(fd, buf, i);
 
