@@ -9,7 +9,7 @@
  *	      report any missing objects (ELF only).
  *        prog ...: programs to check
  *
- * Copyright 1993-1996, David Engel
+ * Copyright 1993-2000, David Engel
  *
  * This program may be used for any purpose as long as this
  * copyright notice is kept.
@@ -133,7 +133,8 @@ int is_bin(char *argv0, char *prog)
 		    }
 		    break;
 		  case ET_DYN:
-		    if ((cp = readsoname(prog, file, LIB_ANY, &libtype)) != NULL)
+		    if ((cp = readsoname(prog, file, LIB_ANY, &libtype, 
+			 elf_hdr.e_ident[EI_CLASS])) != NULL)
 		        free(cp);
 		    if (libtype == LIB_ELF_LIBC5)
 		        res = 5;
@@ -268,13 +269,13 @@ int main(int argc, char **argv, char **envp)
 #endif
 		/* fall through */
 	      case 5: /* ELF libc5 library */
-	        /* if that fials, add library to LD_PRELOAD and 
+	        /* if that fails, add library to LD_PRELOAD and 
 		   then execute lddstub */
 		if (ld_preload && *ld_preload)
-		    snprintf(buff, sizeof buff, "%s=%s:%s%s", "LD_PRELOAD",
+		    snprintf(buff, sizeof buff, "LD_PRELOAD=%s:%s%s", 
 			     ld_preload, *argv[i] == '/' ? "" : "./", argv[i]);
 		else
-		    snprintf(buff, sizeof buff, "%s=%s%s", "LD_PRELOAD", 
+		    snprintf(buff, sizeof buff, "LD_PRELOAD=%s%s", 
 			     *argv[i] == '/' ? "" : "./", argv[i]);
 		putenv(buff);
 		execl(LDDSTUB, argv[i], NULL);
